
unit UnAstRng ; { for different ranges }

interface

uses
  unastcot ; { type tvect3 }

var
  rangeval  : extended ; { value in AU to model range body-Earth }
  azimuval  : extended ; { value in degree to model observations }

{ range from the Earth to the object in AU }

function AstFromEarth ( tc  : Extended ; { moment pos vel AU AU/day }
                        xc  : tvect3 ) : Extended ;  { asteroid }

{ vector from the station to object
  xp position topocentric in terra equa
  xh position topocentric in horizontal }

procedure OstatVector ( tc  : Extended ;      { moment for pos station }
                        xc  : tvect3 ;        { pos fix equa heliocentric }
                        pc  : tcoorec ;       { record for station }
                        var  xp  : tvect3 ;   { vector topo but equ }
                        var  xh  : tvect3 ) ; { vector topo horizon }

{ to obtain position of the Sun
  with known fixed equatorial position
  in topocentric system refer to station }

procedure OsunsVector ( tc  : Extended ;      { moment for pos station }
                        pc  : tcoorec ;       { record for station AU }
                        var  xp  : tvect3 ) ; { vector topo but fix equ }

{ to change
  vector from the station to object
  from terra to fixed equato system }

procedure OstatVequat ( tz  : Extended ;      { moment with delay }
                        xz  : tvect3 ;        { terra equator top }
                        var  xf  : tvect3 ) ; { fixed equator top }

implementation

uses
  unforfun , { simple procedures }
  unfortim , { procedures for TDT TDB }
  unephjpl , { procedures for ephemer }
  unforrom ; { proc fromfixtoterram for matrix }

{ range from the Earth to the object in AU }

function AstFromEarth ( tc  : Extended ; { moment pos vel AU AU/day }
                        xc  : tvect3 ) : Extended ;  { asteroid }
var
  td,dt,tb  : Extended ; { terrestrial time TT barycentric time TDB }
  xe,ve     : tvect3 ; { pos vel for the Earth }
begin
  TransUTCtoTDT(tc,td,dt); { unit unfortim terrestrial time TT }
  TransTDTtoTDB(td,tb);    { barycentric time TDB Teph for ephemeris }
  earths(tb,xe,ve);        { unephjpl the Earth from the Sun vector }
  AstFromEarth:=Sqrt(Sqr(xc[1]-xe[1])   { x position to the Earth AU }
                    +Sqr(xc[2]-xe[2])   { y position to the Earth AU }
                    +Sqr(xc[3]-xe[3])); { z position to the Earth AU }
end;

{ to obtain position of the object
  with known fixed equatorial position
  in topocentric system refer to station }

procedure OstatVector ( tc  : Extended ;      { moment for pos station }
                        xc  : tvect3 ;        { pos fix equa heliocentric }
                        pc  : tcoorec ;       { record for station AU }
                        var  xp  : tvect3 ;   { vector topo but equ }
                        var  xh  : tvect3 ) ; { vector topo horizon }
var
  i   : byte ;     { simple count }
  tb  : Extended ; { barycebtric dynamical time }
  xf  : tvect3 ;   { vector fix equa but geocentric }
  xt  : tvect3 ;   { vector in terra system }
  ft  : TMatr33 ;  { matrix to change from fixed to terra }
  xe  : tvect3 ;   { pos for the Earth heliocentric }
  ve  : tvect3 ;   { vel for the Earth heliocentric }
begin
  { to obtain matrix to change from fixed to terra system }
  fromfixtoterram(tc,0.0,tb,ft); { unforrom to terra matr }
  earths(tb,xe,ve); { unephjpl the Earth from the Sun vector }
  for i:=1 to 3 do      { for three descart positions }
    xf[i]:=xc[i]-xe[i]; { from helio to geocentric AU }
  { to change from fixed to terra system }
  MultMatrVec(ft,xf,xt); { UnForFun vector in terra system AU }
  { to change to topocentric }
  for i:=1 to 3 do { for three descart positions in AU }
    xp[i]:=xt[i]-pc.vauc[i]; { to topocentric but terra system }
  { result to change to horizont station topocentric in AU }  
  MultMatrVec(pc.topm,xp,xh); { UnForFun vector in horizon system }
end;

{ to obtain position of the Sun
  with known fixed ecliptical position refer to station }

procedure OsunsVector ( tc  : Extended ;      { moment for pos station }
                        pc  : tcoorec ;       { record for station AU }
                        var  xp  : tvect3 ) ; { vector topo but fix ecl }
var
  i   : byte ;     { simple count }
  tb  : Extended ; { barycentric dynamical time }
  xt  : tvect3 ;   { vector fix equa for station }
  ft  : TMatr33 ;  { matrix to change from fixed to terra }
  tf  : TMatr33 ;  { matrix to change from terra to fixed }
  xe  : tvect3 ;   { pos for the Earth heliocentric }
  ve  : tvect3 ;   { vel for the Earth heliocentric }
  op  : extended ; { \epsilon_0 }
  se  : extended ; { sin(\epsilon) }
  ce  : extended ; { cos(\epsilon) }
begin
  { to obtain matrix to change from fixed to terra system }
  fromfixtoterram(tc,0.0,tb,ft); { unforrom to terra matr }
  transpmatr(ft,tf); { unforfun matrx from terra to fixed }
  earths(tb,xe,ve); { unephjpl the Earth from the Sun vector }
  { to change from terra to fixed system for station }
  MultMatrVec(tf,pc.vauc,xt); { UnForFun to fixed system AU }
  { to change to topocentric but fixed equator the Sun }
  for i:=1 to 3 do    { for three descart positions in AU }
    xe[i]:=-xe[i]-xt[i]; { to topocentric but fix equator }
  op:=secrad*84381.448;  { \epsilon obliquity in radian }
  se:=sin(op);   { \sin \epsilon obliquity }
  ce:=cos(op);   { \cos \epsilon obliquity }
  xp[1]:=+xe[1]; { rotation around x-axis on angle \epsilon }
  xp[2]:=+ce*xe[2]+se*xe[3]; { ecliptic position }
  xp[3]:=-se*xe[2]+ce*xe[3]; { the Sun to station }
end;

{ to change
  vector from the station to object
  from terra to fixed equato system }

procedure OstatVequat ( tz  : Extended ;      { moment with delay }
                        xz  : tvect3 ;        { terra equator top }
                        var  xf  : tvect3 ) ; { fixed equator top }
var
  tb  : Extended ; { barycentric dynamical time }
  ft  : TMatr33 ;  { matrix to change from fixed to terra }
  tf  : TMatr33 ;  { matrix to change from terra to fixed }
begin
  { to obtain matrix to change from fixed to terra system }
  fromfixtoterram(tz,0.0,tb,ft); { unforrom to terra matr }
  transpmatr(ft,tf); { unforfun matrx from terra to fixed }
  MultMatrVec(tf,xz,xf); { result vector in fixed equator }
end; { procedure OstatVector }
end.
